﻿using System;
using System.Runtime.CompilerServices;
using System.Threading.Tasks;

namespace TestProject
{
    /// <summary>
    /// 텍스트 구하기 비동기 상태 머신
    /// </summary>
    public class GetTextAsyncStateMachine : IAsyncStateMachine
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 프로그램
        /// </summary>
        public Program Program;

        /// <summary>
        /// 비동기 태스크 메소드 빌더
        /// </summary>
        public AsyncTaskMethodBuilder<string> Builder;

        /// <summary>
        /// 상태
        /// </summary>
        public int State;

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 태스크 대기자
        /// </summary>
        private TaskAwaiter<string> taskAwaiter;

        /// <summary>
        /// 문자열 태스크
        /// </summary>
        private Task<string> stringTask;

        /// <summary>
        /// 텍스트
        /// </summary>
        private string text;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region (IAsyncStateMachine) 다음으로 이동하기 - MoveNext()

        /// <summary>
        /// 다음으로 이동하기
        /// </summary>
        public void MoveNext()
        {
            try
            {
                TaskAwaiter<string> taskAwaiter;

                switch(State)
                {
                    case 0 :

                        taskAwaiter = this.taskAwaiter;

                        this.taskAwaiter = new TaskAwaiter<string>();

                        State = -1;

                        break;

                    default :

                        this.stringTask = new TaskFactory().StartNew(() => { return "TEST"; });

                        taskAwaiter = this.stringTask.GetAwaiter();

                        if(taskAwaiter.IsCompleted == false)
                        {
                            State = 0;

                            this.taskAwaiter = taskAwaiter;

                            GetTextAsyncStateMachine asyncStateMachine = this;

                            Builder.AwaitUnsafeOnCompleted(ref taskAwaiter, ref asyncStateMachine);

                            return;
                        }

                        break;
                }

                this.text = taskAwaiter.GetResult();
            }
            catch(Exception exception)
            {
                State = -2;

                Builder.SetException(exception);

                return;
            }

            State = -2;

            Builder.SetResult(this.text);
        }

        #endregion
        #region (IAsyncStateMachine) 상태 머신 설정하기 - SetStateMachine(asyncStateMachine)

        /// <summary>
        /// 상태 머신 설정하기
        /// </summary>
        /// <param name="asyncStateMachine">비동기 상태 머신</param>
        public void SetStateMachine(IAsyncStateMachine asyncStateMachine)
        {
        }

        #endregion
    }
}